"""Define StagingAddOn monkey-patches for CMFUid and Archetypes.

$Id: patches.py 32622 2006-10-31 20:39:30Z dbaty $
"""

## I would prefer monkey-patching OFS.SimpleItem, but Archetypes'
## BaseObject does not call its super class...
from Products.Archetypes.BaseObject import BaseObject

def manage_afterClone(self, item):
    """
    When an item is copy and pasted, we have to make sure that it does
    not contain any staging informations

    See http://plone.org/products/stagingaddon/issues/7
    """
    if getattr(item, 'aq_base', None):
        item = item.aq_base
    for attr in ('_version', '_versions', '_new_version', '_is_a_version_of'):
        if getattr(item, attr, None):
            delattr(item, attr)
    self.orig_manage_afterClone(item)

BaseObject.orig_manage_afterClone = BaseObject.manage_afterClone
BaseObject.manage_afterClone = manage_afterClone


## CMFUid does not allow extra arguments to 'queryBrain()'.
## Unfortunately, this is what we need to list all versions of an
## object. We have to pass extra criteria to the catalog to disable
## default Plone filters (e.g. objects in other languages and expired
## objects).

import logging
from Products.CMFCore.utils import getToolByName

def queryBrain(self, uid, default=None, extra_criteria=None):
    """See IUniqueIdBrainQuery.
    """
    return self._queryBrain(uid, 'searchResults', default,
                            extra_criteria)


def _queryBrain(self, uid, searchMethodName, default=None,
                extra_criteria=None):
    """This helper method does the "hard work" of querying the catalog
    and interpreting the results.

    'extra_criteria' is transferred to the catalog. For example, it
    can be used in Plone to disable the (standard) filtering of
    inactive items, with 'extra_criteria={'show_inactive': True}'.
    """
    if uid is None:
        return default

    # convert the uid to the right format
    generator = getToolByName(self, 'portal_uidgenerator')
    uid = generator.convert(uid)

    if extra_criteria is None:
        extra_criteria = {}
    extra_criteria[self.UID_ATTRIBUTE_NAME] = uid

    catalog = getToolByName(self, 'portal_catalog')
    searchMethod = getattr(catalog, searchMethodName)
    result = searchMethod(**extra_criteria)
    len_result = len(result)

    # return None if no object found with this uid
    if len_result == 0:
        return default

    # print a message to the log  if more than one object has
    # the same uid (uups!)
    if len_result > 1:
        logging.getLogger('CMFUid').error(
            "ASSERT: %d objects have %r as uid!!!", len_result, uid)

    return result[0]


def interface_queryBrain(self, uid, default=None, extra_criteria=None):
    pass


## Patch interface
from Products.CMFUid.interfaces import IUniqueIdBrainQuery
addendum = """
'extra_criteria' is transferred to the catalog. For example, it can be
used in Plone to disable the (standard) filtering of inactive items,
with 'extra_criteria={'show_inactive': True}'.
"""
## FIXME (Damien): the following lines raise an error:
## AttributeError: 'Interface' object has no attribute 'queryBrain'
## I am not sure that there is a solution to that. The Interface
## interface does not seem to provide an accessor to the methods.
#print dir(IUniqueIdBrainQuery)
#print IUniqueIdBrainQuery.names()
#orig_docstring = IUniqueIdBrainQuery['queryBrain'].__doc__
#interface_queryBrain.__doc__ = orig_docstring + addendum
#IUniqueIdBrainQuery['queryBrain'] = interface_queryBrain

## Patch tool
from Products.CMFUid.UniqueIdHandlerTool import UniqueIdHandlerTool
UniqueIdHandlerTool._queryBrain = _queryBrain
UniqueIdHandlerTool.queryBrain = queryBrain
