"""Tests for StagingAddOn installation and uninstallation.

$Id: testInstallation.py 38076 2007-02-19 00:23:41Z dbaty $
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.StagingAddOn.tests.utils import StagingAddOnTestCase

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes, process_types
from Products.StagingAddOn.content.staging_tool import ID, META_TYPE
from Products.StagingAddOn.config import CONFIGLETS, PORTLETS, PROJECT_NAME, SKIN_DIRECTORIES

class TestInstallation(StagingAddOnTestCase):
    """Make sure that the product properly installs and uninstalls."""

    def afterSetUp(self):
        pass


    def testInstallation(self):
        portal = self.portal
        new_types = listTypes(PROJECT_NAME)
        new_types, useless1, useless2 = process_types(new_types,
                                                      PROJECT_NAME)
        self.setRoles(('Manager', ))

        ## Check that skin layers have been added
        skins_tool = getToolByName(portal, 'portal_skins')
        layers = skins_tool.objectIds()
        for skin_directory in SKIN_DIRECTORIES:
            self.failUnless(skin_directory in layers)

        ## Check that skin directories have been added to the skins.
        for skin, layers in skins_tool.getSkinPaths():
            for skin_directory in SKIN_DIRECTORIES:
                self.failUnless(skin_directory in layers)

        ## Check that the tool has been added
        self.failUnless(ID in portal.objectIds())

        ## Check that configlets have been added
        panel = getToolByName(portal, 'portal_controlpanel')
        if panel is not None:
            existing = panel.enumConfiglets(group='Products')
            existing = [c['id'] for c in existing]
            for configlet in CONFIGLETS:
                self.failUnless(configlet['id'] in existing)

        ## Check that portlets have been added
        right_slots = portal.getProperty('right_slots')
        for portlet in PORTLETS:
            self.failUnless(portlet in right_slots)

        ## Check that content types are available
        types_tool = getToolByName(portal, 'portal_types')
        available_types = types_tool.objectIds()
        for new_type in new_types:
            self.failUnless(new_type.portal_type in available_types)

        ## Check that factory types have been set
        factory = getToolByName(portal, 'portal_factory')
        factory_types = factory.getFactoryTypes().keys()
        for new_type in new_types:
            self.failUnless(new_type.portal_type in factory_types)

        ## Check that 'metaTypesNotToList' have been set
        portal_properties = getToolByName(portal, 'portal_properties')
        nav_props = portal_properties.navtree_properties
        metaTypesNotToList = nav_props.getProperty('metaTypesNotToList')
        for new_type in new_types:
            self.failUnless(new_type.portal_type in metaTypesNotToList)

        ## Check that 'types_not_searched' have been set
        site_properties = portal_properties.site_properties
        types_not_searched = site_properties.getProperty('types_not_searched')
        for new_type in new_types:
            self.failUnless(new_type.portal_type in types_not_searched)


    def testIndexationOfNewContentTypes(self):
        portal = self.portal
        ctool = getToolByName(portal, 'portal_catalog')
        stool = getToolByName(portal, ID)
        workflow_id = 'plone_workflow'
        self.createTestUsersAndContent()

        self.login('manager')
        stool.installStagingFor(workflow_id=workflow_id)
        results = ctool.searchResults(id=ID)
        self.failUnless(len(results) == 0)
        results = ctool.searchResults(id=workflow_id)
        self.failUnless(len(results) == 1)


    def testUninstallation(self):
        """Test that the product uninstalls properly."""
        portal = self.portal
        new_types = listTypes(PROJECT_NAME)
        new_types, useless1, useless2 = process_types(new_types,
                                                      PROJECT_NAME)
        self.setRoles(('Manager', ))
        qtool = getToolByName(portal, 'portal_quickinstaller')
        qtool.uninstallProducts(('StagingAddOn', ))

        ## Check that skin layers have been removed from
        ## 'portal_skins' tool.
        skins_tool = getToolByName(portal, 'portal_skins')
        layers = skins_tool.objectIds()
        for skin_directory in SKIN_DIRECTORIES:
            self.failUnless(skin_directory not in layers)

        ## Check that skin directories have been removed from the
        ## skins.
        for skin, layers in skins_tool.getSkinPaths():
            for skin_directory in SKIN_DIRECTORIES:
                self.failUnless(skin_directory not in layers)

        ## Check that the tool has been removed. Configuration items
        ## will therefore be removed, too (and so will be the
        ## corresponding workflow customizations).
        self.failUnless(ID not in portal.objectIds())

        ## Check that configlet have been removed.
        panel = getToolByName(portal, 'portal_controlpanel')
        if panel is not None:
            existing = panel.enumConfiglets(group='Products')
            existing = [c['id'] for c in existing]
            for configlet in CONFIGLETS:
                self.failUnless(configlet['id'] not in existing)

        ## Check that content types are not available anymore
        types_tool = getToolByName(portal, 'portal_types')
        available_types = types_tool.objectIds()
        for new_type in new_types:
            self.failUnless(new_type.portal_type not in available_types)

        ## Check that factory types have been updated
        factory = getToolByName(portal, 'portal_factory')
        factory_types = factory.getFactoryTypes().keys()
        for new_type in new_types:
            self.failUnless(new_type.portal_type not in factory_types)

        ## Check that the portlet has been removed
        right_slots = portal.getProperty('right_slots')
        for portlet in PORTLETS:
            self.failUnless(portlet not in right_slots)

        ## Check that 'metaTypesNotToList' property has been updated
        portal_properties = getToolByName(portal, 'portal_properties')
        nav_props = portal_properties.navtree_properties
        metaTypesNotToList = nav_props.getProperty('metaTypesNotToList')
        for new_type in new_types:
            self.failUnless(new_type.portal_type not in metaTypesNotToList)

        ## Check that 'types_not_searched' property has been updated
        site_properties = portal_properties.site_properties
        types_not_searched = site_properties.getProperty('types_not_searched')
        for new_type in new_types:
            self.failUnless(new_type.portal_type not in types_not_searched)


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    return suite
