"""
Piece of code to demonstrate usage of StagingAddOn and its API

$Id: testUsage.py 38075 2007-02-19 00:22:19Z dbaty $
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.StagingAddOn.tests.utils import StagingAddOnTestCase

from zExceptions import BadRequest
from AccessControl import Unauthorized
from Products.DCWorkflow.Guard import Guard
from Products.CMFCore.utils import getToolByName
from Products.StagingAddOn.content.staging_tool import ID as STOOL_ID
from Products.StagingAddOn.config import WORKFLOW_SCRIPTS, PUBLISH_TRANSITION_POST_SCRIPT_ID, NEW_TRANSITION_ID, NEW_TRANSITION_PROPERTIES

class TestUsage(StagingAddOnTestCase):
    """
    Explain in Python how StagingAddOn works and how to use its API
    """

    def afterSetUp(self):
        """
        Called before each tests

        - create users and content;
        """
        portal = self.portal

        self.createTestUsersAndContent()
        ## We do not need this
        self.login('manager')
        self.portal.folder.manage_delObjects(['subfolder'])


    def testUsage(self):
        """
        Demonstrate what StagingAddOn does and how to use its API
        """
        portal = self.portal
        wtool = getToolByName(portal, 'portal_workflow')
        stool = getToolByName(portal, STOOL_ID)
        folder = portal.folder
        document = folder.document1
        sortFunc = lambda x, y: cmp(x.creation_date,
                                    y.creation_date)

        self.login('manager')

        ## Let's start by installing staging features on a specific
        ## workflow: 'plone_workflow'. There are additional optional
        ## arguments, see 'installStaginFor()' docstring for further
        ## informations.
        stool.installStagingFor(workflow_id='plone_workflow')

        ## We now have staging features installed. It solely involves
        ## a new transition ('createNewVersion') and a script which is
        ## automatically called when the 'publish' transition is
        ## triggered ('promoteVersion').

        ## 'createNewVersion' is available only on 'published'
        ## items. This is how we configured the workflow, see above.
        wtool.doActionFor(document, 'publish')
        wf_actions = wtool.listActions(object=document)
        wf_actions = [a['name'] for a in wf_actions]
        self.failUnless(NEW_TRANSITION_ID not in wf_actions)

        ## Now that we have a new transition, why wouldn't we use it?
        ## "Look, mummy, there are two versions of my document up in
        ## the sky!"
        wtool.doActionFor(document, NEW_TRANSITION_ID)
        folder_items = folder.objectValues()
        folder_items.sort(sortFunc)
        new_version = folder_items[1]

        ## 'new_version' is an exact copy of 'document'. In other
        ## words, 'document' has been literally copied and pasted in
        ## the folder. References have been copied, too (for example:
        ## if 'document' had 'item' in its related items, then
        ## 'new_version' also has 'item' in his related items). Id of
        ## 'new_version' is based on 'document' id: 'foo-new-version'.

        ## So we can assert a few things:
        n = stool.getNewVersion(document)
        self.failUnless(stool.getNewVersion(document) == new_version)
        self.failUnless(stool.hasNewVersion(document) == True)
        self.failUnless(stool.getCurrentVersion(new_version) == document)
        self.failUnless(stool.isCurrentVersion(document) == True)
        self.failUnless(stool.getVersions(document, sortFunc) == [document,
                                                                  new_version])
        ## Note that these methods returns only items which are
        ## viewable by the current user. There also exist private,
        ## unpublished methods to get unrestricted results. See the
        ## code for further details.

        ## We have a new version, which is independent from
        ## 'document'. We can then modify this new version, apply
        ## various workflow transition on it (retract, hide, submit,
        ## etc.). And, eventually, we will want to publish it. This
        ## will automatically swap 'document' and 'new_version'. This
        ## will also rename the old version to something more relevant
        ## ('foo-old-version-replaced-DDMMYYYY-HHMMSS'). And this will
        ## update references to the old version so that these
        ## references will now target the new currently published
        ## version.
        wtool.doActionFor(new_version, 'publish')
        folder_items = folder.objectValues()
        folder_items.sort(sortFunc)
        old_version, current_version = folder_items

        ## We now naturally have:
        self.failUnless(stool.getNewVersion(current_version) is None)
        self.failUnless(not stool.hasNewVersion(current_version))
        self.failUnless(stool.getCurrentVersion(current_version) == current_version)
        self.failUnless(stool.isCurrentVersion(current_version) == True)
        self.failUnless(stool.getVersions(current_version,
                                          sortFunc) == [old_version,
                                                        current_version])

        ## Note that 'old_version' is still published, which is
        ## something you may or may not want. Hopefully, you can
        ## configure StagingAddOn so that the publication of the
        ## new version automatically triggers a specific transition on
        ## the old version. You can also configure the product to
        ## automatically expire the old version.


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestUsage))
    return suite
