"""Define a base class and prepare ZopeTestCase to be used in
StagingAddOn tests.

Vaguely based on RichDocument tests.

$Id: utils.py 33181 2006-11-09 16:06:09Z dbaty $
"""

from Products.StagingAddOn import config
config.TESTING = True

## If LinguaPlone and PloneLanguageTool are installed, we will add
## related tests.
try:
    import Products.LinguaPlone
    import Products.PloneLanguageTool
    HAS_LINGUA_PLONE = True
except ImportError:
    HAS_LINGUA_PLONE = False

## Import the base test case classes
from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase

## Make ZopeTestCase aware of StagingAddOn
ZopeTestCase.installProduct('StagingAddOn')
if HAS_LINGUA_PLONE:
    ZopeTestCase.installProduct('LinguaPlone')
    ZopeTestCase.installProduct('PloneLanguageTool')

## Set up a Plone site
PRODUCTS = ['StagingAddOn']
if HAS_LINGUA_PLONE:
    PRODUCTS.extend(('LinguaPlone', 'PloneLanguageTool'))
PloneTestCase.setupPloneSite(products=PRODUCTS)
PloneTestCase.installProduct('Five')


## Import statements
from Products.CMFCore.utils import getToolByName
from Products.StagingAddOn.content.staging_tool import ID as STOOL_ID


class StagingAddOnTestCase(PloneTestCase.PloneTestCase):
    """
    Base class for StagingAddOn tests
    """
    def createTestUsersAndContent(self):
        """
        Create some test users and content

        Four users are created:
        - a manager: 'manager'
        - a reviewer: 'reviewer'
        - two members: 'member1' and 'member2'

        The following items are created:
        + folder
        |--+-- document1
        |--+-- subfolder
        |-----+-- document2
        |-----+-- document3
        + Members
        |--+-- member1
        |-----+-- document
        """
        portal = self.portal

        ## Create users
        mtool = getToolByName(portal, 'portal_membership')
        users = ({'id': 'member1', 'roles': ('Member', )},
                 {'id': 'member2', 'roles': ('Member', )},
                 {'id': 'reviewer', 'roles': ('Reviewer', )},
                 {'id': 'manager', 'roles': ('Manager', )})
        for user in users:
            mtool.addMember(user['id'], user['id'], user['roles'], [],
                            properties={})

        ## Create content
        self.login('manager')
        portal.invokeFactory('Folder', 'folder')
        folder = portal.folder
        folder.invokeFactory('Document', 'document1')
        folder.invokeFactory('Folder', 'subfolder')
        subfolder = folder.subfolder
        subfolder.invokeFactory('Document', 'document2')
        subfolder.invokeFactory('Document', 'document3')
        portal.portal_membership.createMemberArea('member1')
        self.login('member1')
        portal.Members.member1.invokeFactory('Document', 'document')

        self.logout()
